/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#include <memory.h> /* for memset */
#include "rquota.h"

/* Default timeout can be changed using clnt_control() */
static struct timeval TIMEOUT = { 25, 0 };

getquota_rslt *
rquotaproc_getquota_1(getquota_args *argp, CLIENT *clnt)
{
	static getquota_rslt clnt_res;

	memset((char *)&clnt_res, 0, sizeof(clnt_res));
	if (clnt_call (clnt, RQUOTAPROC_GETQUOTA,
		(xdrproc_t) xdr_getquota_args, (caddr_t) argp,
		(xdrproc_t) xdr_getquota_rslt, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

getquota_rslt *
rquotaproc_getactivequota_1(getquota_args *argp, CLIENT *clnt)
{
	static getquota_rslt clnt_res;

	memset((char *)&clnt_res, 0, sizeof(clnt_res));
	if (clnt_call (clnt, RQUOTAPROC_GETACTIVEQUOTA,
		(xdrproc_t) xdr_getquota_args, (caddr_t) argp,
		(xdrproc_t) xdr_getquota_rslt, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

setquota_rslt *
rquotaproc_setquota_1(setquota_args *argp, CLIENT *clnt)
{
	static setquota_rslt clnt_res;

	memset((char *)&clnt_res, 0, sizeof(clnt_res));
	if (clnt_call (clnt, RQUOTAPROC_SETQUOTA,
		(xdrproc_t) xdr_setquota_args, (caddr_t) argp,
		(xdrproc_t) xdr_setquota_rslt, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

setquota_rslt *
rquotaproc_setactivequota_1(setquota_args *argp, CLIENT *clnt)
{
	static setquota_rslt clnt_res;

	memset((char *)&clnt_res, 0, sizeof(clnt_res));
	if (clnt_call (clnt, RQUOTAPROC_SETACTIVEQUOTA,
		(xdrproc_t) xdr_setquota_args, (caddr_t) argp,
		(xdrproc_t) xdr_setquota_rslt, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

getquota_rslt *
rquotaproc_getquota_2(ext_getquota_args *argp, CLIENT *clnt)
{
	static getquota_rslt clnt_res;

	memset((char *)&clnt_res, 0, sizeof(clnt_res));
	if (clnt_call (clnt, RQUOTAPROC_GETQUOTA,
		(xdrproc_t) xdr_ext_getquota_args, (caddr_t) argp,
		(xdrproc_t) xdr_getquota_rslt, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

getquota_rslt *
rquotaproc_getactivequota_2(ext_getquota_args *argp, CLIENT *clnt)
{
	static getquota_rslt clnt_res;

	memset((char *)&clnt_res, 0, sizeof(clnt_res));
	if (clnt_call (clnt, RQUOTAPROC_GETACTIVEQUOTA,
		(xdrproc_t) xdr_ext_getquota_args, (caddr_t) argp,
		(xdrproc_t) xdr_getquota_rslt, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

setquota_rslt *
rquotaproc_setquota_2(ext_setquota_args *argp, CLIENT *clnt)
{
	static setquota_rslt clnt_res;

	memset((char *)&clnt_res, 0, sizeof(clnt_res));
	if (clnt_call (clnt, RQUOTAPROC_SETQUOTA,
		(xdrproc_t) xdr_ext_setquota_args, (caddr_t) argp,
		(xdrproc_t) xdr_setquota_rslt, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}

setquota_rslt *
rquotaproc_setactivequota_2(ext_setquota_args *argp, CLIENT *clnt)
{
	static setquota_rslt clnt_res;

	memset((char *)&clnt_res, 0, sizeof(clnt_res));
	if (clnt_call (clnt, RQUOTAPROC_SETACTIVEQUOTA,
		(xdrproc_t) xdr_ext_setquota_args, (caddr_t) argp,
		(xdrproc_t) xdr_setquota_rslt, (caddr_t) &clnt_res,
		TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&clnt_res);
}
